package GII::Event::Command;

=head1 NAME

GII::Event::Command - Perl extension for LibGII command events.

=head1 SYNOPSIS

  use GII::Event::Command;

  $cmd = new GII::Event::Command;
  $cmd = new GII::Event::Command $type;
  $cmd = new GII::Event::Command $type $obj;
  $cmd = new GII::Event::Command $type $obj $code;

  $code = $cmd->code;
  $cmd->code($code);

  $data_size = $cmd->data_size;
  $cmd->data_size($data_size);

  $data = $cmd->data;
  $cmd->data($data);

  $input->send($cmd);

=head1 DESCRIPTION

GII::Event::Command objects subclass GII::Event objects and 
add the methods necessary to manipulate LibGII gii_cmd_event
and gii_cmd_nodata_event structures.  

=cut 

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %ClassMap);
use GII::Event;

require Exporter;
require DynaLoader;
require AutoLoader;

=head1 Exported Constants

CMDFLAG_NODATA
CMDFLAG_PRIVATE
CMDFLAG_EXTERNAL
CMD_DATA_MAX
CMDCODE_EVENTLOST
CMDCODE_GETDEVINFO
CMDCODE_GETVALINFO

=cut

@ISA = qw(Exporter AutoLoader DynaLoader GII::Event);
@EXPORT = qw(
CMDFLAG_NODATA
CMDFLAG_PRIVATE
CMDFLAG_EXTERNAL 	
CMD_DATA_MAX
CMDCODE_EVENTLOST
CMDCODE_GETDEVINFO
CMDCODE_GETVALINFO
);
$VERSION = '0.01';

bootstrap GII::Event::Command $VERSION;

=head1 Global Variable

The class also serves as a superclass for specific types 
of commands.  As such, it also supplies a global hash, mapping 
LibGII command codes to to object classes, named 
%GII::Event::Command::ClassMap.  This is used to choose the right 
subclass in which to place an event received by a GII:: object or 
created by the user.

Subclasses not distributed with the base GII:: modules are welcome to
modify this hash to hook their class into this automagical mechanism;
but must store/restore the previously existing value if there was one.

=cut

%ClassMap = (
	     &CMDCODE_GETDEVINFO => 'GII::Event::Command::DevInfo',
	     &CMDCODE_GETVALINFO => 'GII::Event::Command::ValInfo'
	     );

=head1 Methods

A GII::Event::Command has the following methods beyond, or differing 
from, the ones inherited from the GII::Event superclass:

=head2 new

The GII::Event::Command constructor creates a gii_cmd_event structure 
when called with no arguments, but initializes it to contain only a 
gii_cmd_nodata_event structure.  If given arguments, the 
first is assumed to be an event type.  The second, if present and
defined, is a pointer to a pre-allocated ggi_event which will be used
instead of a newly allocated structure.  The third is assumed to
be a command code, and is used to find a subclass constructor if one 
is present for that command code.  If the first argument is undef, 
then the object is assumed to contain valid event data already and 
the subclass is chosen from the command code therein (the third 
argument would be ignored in this case if given.)  All of the arguments 
are passed along to any subclass constructors that are invoked.

=cut

sub new {
    my ($class) = shift();
    my($type) = shift();
    my($obj) = shift();
    my($code);
    $code = shift() if defined($type);
    if (defined($code)) {
	return $ClassMap{$code}->new($type, $obj, $code, @_)
	    unless ($ClassMap{$code} eq $class);
    }
    else {
	$code = &CMDFLAG_NODATA;
    }
    my($this) = {};	
    bless $this, $class;
    if (not defined($obj) or defined($type)) {
	$this->{obj} = defined($obj) ? $obj : $this->_alloc;
	$this->set_type($this->{obj}, defined($type) ? $type : &evCommand);
	$this->set_code($this->{obj}, $code);
	$this->set_size($this->{obj}, 0);
	$this->set_data_size($this->{obj}, 0);
	return($this);
    }
    $this->{obj} = $obj;
    $code = $this->code;
    unless ($ClassMap{$code} eq $class) {
	delete($this->{obj}); # avoid free() in DESTROY
	return $ClassMap{$code}->new($type, $obj, $code, @_);
    }
    return($this);
}

# DESTROY is inherited

=head2 code

The code method with no arguments returns the command code, as per the 
above exported constants (subclasses may have additional codes.)
With an argument it sets the code, and returns the previous code.

=cut

sub code {
    my($this) = shift();
    if (@_ + 0) {
        return set_code($this->{obj}, shift());
    }
    return get_code($this->{obj});
}

=head2 data

The data method with no arguments returns the data contained in 
the command event.   With a single argument it loads the command 
event with the given data (interpreted as a perl scalar in string 
context), and returns the previous code.

Data is stored/retrieved up to the number of bytes in the 
data buffer (see the data_size method below), and will be 
null-padded to this length when stored/retrieved.

A third argument will allow you to load data and set the data_size 
value (see below) at the same time.  If the argument is undef,
the data_size value is automatically set to the length of the 
data.  Otherwise it is set to the value of the argument.  The
above restrictions still apply, of course.

=cut

sub data {
    my($this) = shift();
    if (@_ + 0) {
	my ($data) = shift();
	my($size);
	if (@_ + 0) {
	    my($data_size) = shift();
	    $data_size = length($data) unless defined ($data_size);
	    set_data_size($this->{obj},$data_size);
	}
        return set_data($this->{obj}, $size);
    }
    return get_data($this->{obj});
}

=head2 data_size

The data_size method with no arguments returns the size of the 
data buffer contained in the command event.   With an argument 
it sets the size of the data buffer and returns the old size.  
This affects the value of the size method inherited from the 
GII::Event class.  The constant CMD_DATA_MAX defines the maximum 
number of bytes the buffer may contain.  Request for more than this 
number of bytes will set the buffer size to CMD_DATA_MAX.
If you invoke this method on an object containing a 
gii_cmd_nodata_event structure you'll get what you deserve.
Only use it on objects you created yourself (not those returned
from GII:: object methods) unless you know exactly what you are doing.

=cut

sub data_size {
    my($this) = shift();
    if (@_ + 0) {
        return set_data_size($this->{obj}, shift());
    }
    return get_data_size($this->{obj});
}


1;
__END__

=head1 AUTHOR

Brian S. Julin <bri@calyx.com>

=head1 SEE ALSO

GII::Event(3pm), GII::(3pm), perl(1).

=cut
